/* epicsEvent */
#ifndef epicsEventh
#define epicsEventh

#include <vxWorks.h>
#include <taskLib.h>
#include <semLib.h>

#define epicsEventId SEM_ID
typedef enum {
    epicsEventWaitOK,epicsEventWaitTimeout,epicsEventWaitError
} epicsEventWaitStatus;

typedef enum {epicsEventEmpty,epicsEventFull} epicsEventInitialState;

#define epicsEventCreate(state)  \
    semBCreate(SEM_Q_FIFO, state)

#define epicsEventMustCreate(state) __extension__ \
({ \
    SEM_ID e = epicsEventCreate(state); \
    if (!e) { printErr("FATAL: epicsEventMustCreate failed in %s line %d: %s\n", \
        __FILE__, __LINE__, strerror(errno)); taskSuspend(0); } \
    e; \
})

#define epicsEventWait(e) \
    (semTake((e), WAIT_FOREVER) == OK ? epicsEventWaitOK : \
    epicsEventWaitError)

#define epicsEventWaitWithTimeout(e,t) \
    (semTake((e), t*sysClkRateGet()) == OK ? epicsEventWaitOK : \
        errno == S_objLib_OBJ_UNAVAILABLE ? epicsEventWaitTimeout : \
        epicsEventWaitError)

#define epicsEventMustWait(e) \
    (semTake(e, WAIT_FOREVER) && \
    (printErr("FATAL: epicsEventMustWait failed in %s line %d: %s\n", \
        __FILE__, __LINE__, strerror(errno)), \
    taskSuspend(0), ERROR))

#define epicsEventTryWait(e) \
    (semTake((e), NO_WAIT ? epicsEventWaitOK : \
        errno == S_objLib_OBJ_UNAVAILABLE ? epicsEventWaitTimeout : \
        epicsEventWaitError)
     
#define epicsEventSignal semGive
#define epicsEventDestroy semDelete

#endif
