/* epicsMutex */
#ifndef epicsMutexh
#define epicsMutexh

#include <vxWorks.h>
#include <taskLib.h>
#include <semLib.h>

#ifdef __cplusplus
extern "C" {
#endif /*__cplusplus*/


#define epicsMutexId SEM_ID

typedef enum {
    epicsMutexLockOK,epicsMutexLockTimeout,epicsMutexLockError
} epicsMutexLockStatus;

#define epicsMutexCreate() \
    semMCreate(SEM_DELETE_SAFE|SEM_INVERSION_SAFE|SEM_Q_PRIORITY);

#define epicsMutexMustCreate() __extension__ \
({ \
    SEM_ID m = epicsMutexCreate(); \
    if (!m) { printErr("FATAL: epicsMutexMustCreate failed in %s line %d: %s\n", \
        __FILE__, __LINE__, strerror(errno)); taskSuspend(0); } \
    m; \
})

#define epicsMutexLock(m) \
    ((semTake(m, WAIT_FOREVER) == OK) ? epicsMutexLockOK : \
        epicsMutexLockError)

#define epicsMutexMustLock(m) \
    (semTake(m, WAIT_FOREVER) && \
    (printErr("FATAL: epicsMutexMustLock(%p) failed in %s line %d: %s\n", \
        (m), __FILE__, __LINE__, strerror(errno)), \
    taskSuspend(0), ERROR))

#define epicsMutexTryLock(m) \
    ((semTake(m, NO_WAIT) == OK) ? epicsMutexLockOK : \
        errno == S_objLib_OBJ_UNAVAILABLE ? epicsMutexLockTimeout : \
        epicsMutexLockError)

#define epicsMutexUnlock semGive
#define epicsMutexDestroy semDelete

#ifdef __cplusplus
}
#endif

#endif
